class TesseractOCRPipelineModelList extends ResourceList {
  static autoInit() {
    for (let tesseractOCRPipelineModelListElement of document.querySelectorAll('.tesseract-ocr-pipeline-model-list:not(.no-autoinit)')) {
      new TesseractOCRPipelineModelList(tesseractOCRPipelineModelListElement);
    }
  }

  constructor(listContainerElement, options = {}) {
    super(listContainerElement, options);
    this.listjs.list.addEventListener('change', (event) => {this.onChange(event)});
    this.listjs.list.addEventListener('click', (event) => {this.onClick(event)});
    this.isInitialized = false;
    this.userId = listContainerElement.dataset.userId;
    app.subscribeUser(this.userId).then((response) => {
      app.socket.on('PATCH', (patch) => {
        if (this.isInitialized) {this.onPatch(patch);}
      });
    });
    app.getUser(this.userId).then((user) => {
      this.add(Object.values(user.tesseract_ocr_pipeline_models));
      for (let uncheckedCheckbox of this.listjs.list.querySelectorAll('input[data-checked="True"]')) {
        uncheckedCheckbox.setAttribute('checked', '');
      }
      if (user.role.name !== ('Administrator' || 'Contributor')) {
        for (let switchElement of this.listjs.list.querySelectorAll('.is_public')) {
          switchElement.setAttribute('disabled', '');
        }
      }
      this.isInitialized = true;
    });
  }

  get item() {
    return `
      <tr class="list-item clickable hoverable">
        <td><b><span class="title"></span> <span class="version"></span></b><br><i><span class="description"></span></i></td>
        <td><a class="publisher-url"><span class="publisher"></span></a> (<span class="publishing-year"></span>)<br><a class="publishing-url"><span class="publishing-url-2"></span></a></td>
        <td>
          <div class="list-action-trigger switch center-align" data-list-action="share-request">
            <span class="share"></span>
            <label>
              <input type="checkbox" class="is_public">
              <span class="lever"></span>
              public
            </label>
          </div>
        </td>
        <td class="right-align">
          <a class="list-action-trigger btn-floating red waves-effect waves-light" data-list-action="delete-request"><i class="material-icons">delete</i></a>
          <a class="list-action-trigger btn-floating service-color darken waves-effect waves-light service-2" data-list-action="view"><i class="material-icons">send</i></a>
        </td>
      </tr>
    `.trim();
  }

  get valueNames() {
    return [
      {data: ['id']},
      {data: ['creation-date']},
      {name: 'publisher-url', attr: 'href'},
      {name: 'publishing-url', attr: 'href'},
      'description',
      'publisher',
      'publishing-url-2',
      'publishing-year',
      'title',
      'title-2',
      'version',
      {name: 'is_public', attr: 'data-checked'}
    ];
  }

  initListContainerElement() {
    if (!this.listContainerElement.hasAttribute('id')) {
      this.listContainerElement.id = Utils.generateElementId('tesseract-ocr-pipeline-model-list-');
    }
    let listSearchElementId = Utils.generateElementId(`${this.listContainerElement.id}-search-`);
    this.listContainerElement.innerHTML = `
      <div class="input-field">
        <i class="material-icons prefix">search</i>
        <input id="${listSearchElementId}" class="search" type="text"></input>
        <label for="${listSearchElementId}">Search Tesseract OCR Pipeline Model</label>
      </div>
      <table>
        <thead>
          <tr>
            <th>Title and Description</th>
            <th>Publisher</th>
            <th></th>
          </tr>
        </thead>
        <tbody class="list"></tbody>
      </table>
      <ul class="pagination"></ul>
    `.trim();
  }

  mapResourceToValue(tesseractOCRPipelineModel) {
    return {
      'id': tesseractOCRPipelineModel.id,
      'creation-date': tesseractOCRPipelineModel.creation_date,
      'description': tesseractOCRPipelineModel.description,
      'publisher': tesseractOCRPipelineModel.publisher,
      'publisher-url': tesseractOCRPipelineModel.publisher_url,
      'publishing-url': tesseractOCRPipelineModel.publishing_url,
      'publishing-url-2': tesseractOCRPipelineModel.publishing_url,
      'publishing-year': tesseractOCRPipelineModel.publishing_year,
      'title': tesseractOCRPipelineModel.title,
      'title-2': tesseractOCRPipelineModel.title,
      'version': tesseractOCRPipelineModel.version,
      'is_public': tesseractOCRPipelineModel.is_public ? 'True' : 'False'
    };
  }

  sort() {
    this.listjs.sort('creation-date', {order: 'desc'});
  }

  onChange(event) {
    let actionSwitchElement = event.target.closest('.list-action-trigger');
    let action = actionSwitchElement.dataset.listAction;
    let tesseractOCRPipelineModelElement = event.target.closest('tr');
    let tesseractOCRPipelineModelId = tesseractOCRPipelineModelElement.dataset.id;
    switch (action) {
      case 'share-request': {
        let is_public = actionSwitchElement.querySelector('input').checked;
        Utils.shareTesseractOCRPipelineModelRequest(this.userId, tesseractOCRPipelineModelId, is_public);
        break;
      }
      default: {
        break;
      }
    }
  }

  onClick(event) {
    if (event.target.closest('.action-switch')) {
      let userRole = app.data.users[this.userId].role.name;
      if (userRole !== ('Administrator' || 'Contributor')) {
        app.flash('You need the "Contributor" or "Administrator" role to perform this action.', 'error');
      }
      return;
    }
    let actionButtonElement = event.target.closest('.list-action-trigger');
    let action = actionButtonElement === null ? 'view' : actionButtonElement.dataset.listAction;
    let tesseractOCRPipelineModelElement = event.target.closest('tr');
    let tesseractOCRPipelineModelId = tesseractOCRPipelineModelElement.dataset.id;
    switch (action) {
      case 'delete-request': {
        Utils.deleteTesseractOCRPipelineModelRequest(this.userId, tesseractOCRPipelineModelId);
        break;
      }
      case 'view': {
        window.location.href = `/contributions/tesseract-ocr-pipeline-models/${tesseractOCRPipelineModelId}`;
        break;
      }
      default: {
        break;
      }
    }
  }
}
